-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

local L = RaidChecklistStrings
local P = RaidChecklistPetFamilies

RaidChecklistData = {}
local data = RaidChecklistData

--
-- Primary ability categories
--
data.buffs = {
  "CRIT", "ATTACKPWR", "SPELLPWR",
  "PHYSHASTE", "SPELLHASTE",
  "STATS", "STAM",
  "MASTERY", "BURSTMANA", "BLOODLUST"
}

data.debuffs = {
  "PHYSDMGTAKEN", "SPELLDMGTAKEN",
  "CASTSPD", "WEAKBLOWS",
  "SUNDER", "MORTALSTRIKE",
}

data.dispels = {
  "OFFMAGIC", "DEFMAGIC",
  "DISEASE", "POISON",
  "CURSE", "ENRAGE"
}

data.ccs = {
  "CCBEASTS", "CCDEMONS", "CCDRAGONKIN", "CCELEMENTALS", "CCGIANTS", "CCHUMANOIDS", "CCUNDEAD", "CCANY",
  "FEARBEASTS", "FEARUNDEAD", "FEARANY"
}

data.miscs = {
  "KNOCKBACK", "INTERRUPT", "INTERRUPT2", "SNARE", "AOESNARE", "ROOT", "DISARM"
}


--
-- Secondary ability categories.
--
-- All abilities in these categories here are already covered by the primary categories.
-- These groupings exist purely for display groupings.
--

data.hunterpetabilities = {
  "CRIT", "STATS", "STAM", "BLOODLUST",
  "PHYSDMGTAKEN", "SPELLDMGTAKEN", "CASTSPD", "WEAKBLOWS", "SUNDER", "MORTALSTRIKE",
  "INTERRUPT", "INTERRUPT2", "DISARM"
}

data.warlockpetabilities = {
  "STAM", "MANA", "MANAREGEN",
  "MORTALSTRIKE",
  "CCHUMANOIDS",
  "KNOCKBACK", "INTERRUPT"
}

data.totemsall = {
  "AOESNARE",
  "SPELLPWR",
  "MANAREGEN",
  "PHYSHASTE", "SPELLHASTE",
}


-- Abuse the metatable functionality in order to make sure any abilities referenced by the providers actually exists.
local ability_typo_checker = { __index = function (table, key)
  for i,b in ipairs (table) do
    if b == key then return key end
  end
  DEFAULT_CHAT_FRAME:AddMessage ("|cffff0000>> RaidChecklist: unknown ability name: |r"..(key or "nil"))
end
}
setmetatable (data.buffs, ability_typo_checker)
setmetatable (data.debuffs, ability_typo_checker)
setmetatable (data.dispels, ability_typo_checker)
setmetatable (data.ccs, ability_typo_checker)
setmetatable (data.miscs, ability_typo_checker)

-- Convenience function to create a "provider" table
data.create_provider_table = function (self)
  local t = {}
  for i,b in ipairs (self.buffs)   do t[b] = {} end
  for i,b in ipairs (self.debuffs) do t[b] = {} end
  for i,b in ipairs (self.dispels) do t[b] = {} end
  for i,b in ipairs (self.ccs)     do t[b] = {} end
  for i,b in ipairs (self.miscs)   do t[b] = {} end
  return t
end


--
-- Core ability table.
-- Malazee rocks for compiling all this information and keeping it updated!
--

local B = data.buffs
local D = data.debuffs
local S = data.dispels
local C = data.ccs
local M = data.miscs

local R = { BLOODELF = "BloodElf" } -- "Filename" from UnitRace(), locale-independent

local SPEC = " "
local NOTE = "*" -- Indicates a talent which is accessible regardless of talent spec

-- Legend:
--   a - Ability
--   c - Class (localized)
--   s - Spec name (localized)
--   p - Pet family (localized)
--   l - Level requirement
--   tr - Talent row
--   tc - Talent column
--   n - Talent name, automatically looked up from 'i'
--   i - Spell/ability ID
--   e - Race-specific ability
--   display - Text to display for this entry
data.table = {
  { a = S.CURSE,          c = L.DRUID,        s = L.BALANCE,                          l = 24,                i = 2782                                                                   },
  { a = S.CURSE,          c = L.DRUID,        s = L.RESTODRUID,                                   t = 88423                                                                             },
  { a = S.CURSE,          c = L.MAGE,                                                 l = 30,                i = 475                                                                    },
  { a = S.CURSE,          c = L.SHAMAN,                                               l = 18,                i = 51886                                                                  },

  { a = S.DISEASE,        c = L.PALADIN,                                              l = 34,                i = 4987                                                                   },
  { a = S.DISEASE,        c = L.PRIEST,       s = L.HOLYPRIEST,                       l = 22,                i = 528                                                                    },
  { a = S.DISEASE,        c = L.PRIEST,       s = L.DISCIPLINE,                       l = 22,                i = 528                                                                    },

  { a = S.ENRAGE,         c = L.DRUID,        s = L.BALANCE,                          l = 28,                i = 2908                                                                   },
  { a = S.ENRAGE,         c = L.DRUID,        s = L.RESTODRUID,                       l = 28,                i = 2908                                                                   },
  { a = S.ENRAGE,         c = L.HUNTER,                                               l = 35,                i = 19801                                                                  },
  { a = S.ENRAGE,         c = L.ROGUE,                                                l = 70,                i = 5938                                                                   },

  { a = S.DEFMAGIC,       c = L.DRUID,        s = L.RESTODRUID,                                   t = 88423                                                                             },
  { a = S.DEFMAGIC,       c = L.PALADIN,      s = L.HOLYPALADIN,                                  t = 53551                                                                             },
  { a = S.DEFMAGIC,       c = L.PRIEST,       s = L.HOLYPRIEST,                       l = 26,                i = 527                                                                    },
  { a = S.DEFMAGIC,       c = L.PRIEST,       s = L.DISCIPLINE,                       l = 26,                i = 527                                                                    },
  { a = S.DEFMAGIC,       c = L.SHAMAN,       s = L.RESTOSHAMAN,                                  t = 77130                                                                             },

  { a = S.OFFMAGIC,       c = L.HUNTER,                                               l = 35,                i = 19801                                                                  },
  { a = S.OFFMAGIC,       c = L.MAGE,                                                 l = 64,                i = 30449                                                                  },
  { a = S.OFFMAGIC,       c = L.SHAMAN,                                               l = 12,                i = 370                                                                    },

  { a = S.POISON,         c = L.DRUID,        s = L.BALANCE,                          l = 24,                i = 2782                                                                   },
  { a = S.POISON,         c = L.DRUID,        s = L.RESTODRUID,                                   t = 88423                                                                             },
  { a = S.POISON,         c = L.PALADIN,                                              l = 18,                i = 4987                                                                   },

  -- checked from here:
  { a = B.STAM,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SILITHID,     l = 4,                 i = 90364                                                                  },
  { a = B.STAM,           c = L.PRIEST,                                               l = 22,                i = 21562                                                                  },
  { a = B.STAM,           c = L.WARLOCK,                          p = P.IMP,          l = 4,                 i = 103127                                                                 },
  { a = B.STAM,           c = L.WARRIOR,                                              l = 68,                i = 469                                                                    },

  { a = B.BLOODLUST,      c = L.HUNTER,       s = L.BEASTMASTER,  p = P.COREHOUND,    l = 20,                i = 90355                                                                  },
  { a = B.BLOODLUST,      c = L.MAGE,                                                 l = 85,                i = 80353                                                                  },
  { a = B.BLOODLUST,      c = L.SHAMAN,                                               l = 70,                i = 2825                                                                   }, -- and 32182 (Heroism)

  { a = B.BURSTMANA,      c = L.DRUID,        s = L.BALANCE,                          l = 54,                i = 29166                                                                  },
  { a = B.BURSTMANA,      c = L.DRUID,        s = L.RESTODRUID,                       l = 54,                i = 29166                                                                  },
  { a = B.BURSTMANA,      c = L.PRIEST,                                               l = 64,                i = 64904                                                                  },
  { a = B.BURSTMANA,      c = L.SHAMAN,       s = L.RESTOSHAMAN,                      l = 56,                i = 16190                                                                  },

  { a = B.ATTACKPWR,      c = L.DEATHKNIGHT,                                          l = 65,                i = 57330                                                                  },
  { a = B.ATTACKPWR,      c = L.HUNTER,                                               l = 39,                i = 19506                                                                  },
  { a = B.ATTACKPWR,      c = L.WARRIOR,                                              l = 42,                i = 6673                                                                   },

  { a = B.CRIT,           c = L.DRUID,        s = L.FERAL,                            l = 46,     t = 17007                                                                             },
  { a = B.CRIT,           c = L.DRUID,        s = L.GUARDIAN,                         l = 46,     t = 17007                                                                             },
  { a = B.CRIT,           c = L.HUNTER,                           p = P.WOLF,         l = 20,                i = 24604                                                                  },
  { a = B.CRIT,           c = L.HUNTER,                           p = P.HYDRA,        l = 20,                i = 97229                                                                  },
  { a = B.CRIT,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.DEVILSAUR,    l = 20,                i = 90309                                                                  },
  { a = B.CRIT,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.QUILEN,       l = 20,                i = 126373                                                                 },
  { a = B.CRIT,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.WATERSTRIDER, l = 20,                i = 126309                                                                 },
  { a = B.CRIT,           c = L.MAGE,                                                 l = 58,                i = 1459                                                                   },
  { a = B.CRIT,           c = L.MONK,         s = L.WINDWALKER,                       l = 81,     i = 116781                                                                            },

  { a = B.PHYSHASTE,      c = L.DEATHKNIGHT,  s = L.FROSTDK,                          l = 60,                i = 55610                                                                  },
  { a = B.PHYSHASTE,      c = L.DEATHKNIGHT,  s = L.UNHOLY,                           l = 60,                i = 55610                                                                  },
  { a = B.PHYSHASTE,      c = L.HUNTER,                           p = P.HYENA,        l = 20,                i = 128432                                                                 },
  { a = B.PHYSHASTE,      c = L.HUNTER,                           p = P.SERPENT,      l = 20,                i = 128433                                                                 },
  { a = B.PHYSHASTE,      c = L.ROGUE,                                                l = 30,                i = 113742                                                                 },
  { a = B.PHYSHASTE,      c = L.SHAMAN,       s = ENHANCEMENT,                        l = 55,     t = 30809                                                                             },

  { a = B.SPELLHASTE,     c = L.DRUID,        s = L.BALANCE,                          l = 16,     t = 24858                                                                             },
  { a = B.SPELLHASTE,     c = L.PRIEST,       s = L.SHADOW,                           l = 24,     t = 15473                                                                             },
  { a = B.SPELLHASTE,     c = L.SHAMAN,       s = L.ELEMENTAL,                        l = 55,                i = 51470                                                                  },

  { a = B.SPELLPWR,       c = L.MAGE,                                                 l = 58,                i = 1459                                                                   },
  { a = B.SPELLPWR,       c = L.SHAMAN,                                               l = 40,                i = 77747                                                                  },
  { a = B.SPELLPWR,       c = L.WARLOCK,                                              l = 82,                i = 109773                                                                 },
  { a = B.SPELLPWR,       c = L.HUNTER,       s = L.BEASTMASTER,  p = P.WATERSTRIDER, l = 20,                i = 126309                                                                 },

  { a = B.STATS,          c = L.DRUID,                                                l = 62,                i = 1126                                                                   },
  { a = B.STATS,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SHALESPIDER,  l = 20,                i = 90363                                                                  },
  { a = B.STATS,          c = L.MONK,                                                 l = 22,                i = 115921                                                                 },
  { a = B.STATS,          c = L.PALADIN,                                              l = 30,                i = 20217                                                                  },

  { a = B.MASTERY,        c = L.PALADIN,                                              l = 81,                i = 19740                                                                  },
  { a = B.MASTERY,        c = L.SHAMAN,                                               l = 80,                i = 116956                                                                 },
  { a = B.MASTERY,        c = L.HUNTER,                           p = P.CAT,          l = 20,                i = 93435                                                                  },
  { a = B.MASTERY,        c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SPIRITBEAST,  l = 20,                i = 128997                                                                 },
  -- checked to here

  { a = M.DISARM,         c = L.HUNTER,                           p = P.SCORPID,                             i = 50541                                                                  },
  { a = M.DISARM,         c = L.HUNTER,                           p = P.BIRDOFPREY,                          i = 91644                                                                  },
  { a = M.DISARM,         c = L.PRIEST,       s = L.SHADOW,                                       t = 64044                                                                             },
  { a = M.DISARM,         c = L.ROGUE,                                                l = 38,                i = 51722                                                                  },
  { a = M.DISARM,         c = L.WARRIOR,                                              l = 34,                i = 676                                                                    },

  { a = C.FEARBEASTS,     c = L.HUNTER,                                               l = 36,                i = 1513                                                                   },
  { a = C.FEARUNDEAD,     c = L.PALADIN,                                              l = 78,                i = 10326                                                                  }, -- Note: also included Demons!
  { a = C.FEARANY,        c = L.PRIEST,                                                         tr=1,tc=2,   i = 108921                                                                 },
  { a = C.FEARANY,        c = L.WARLOCK,                                              l = 14,                i = 5782                                                                   },
  { a = C.FEARANY,        c = L.WARLOCK,                                                        tr=4,tc=1,   i = 111397                                                                 },

  { a = M.INTERRUPT,      c = L.DEATHKNIGHT,                                          l = 57,                i = 47528                                                                  },
  { a = M.INTERRUPT,      c = L.DRUID,        s = L.FERAL,                            l = 64,     t = 106839                                                                            },
  { a = M.INTERRUPT,      c = L.HUNTER,                           p = P.GORILLA,                             i = 26090                                                                  },
  { a = M.INTERRUPT,      c = L.MAGE,                                                 l = 9,                 i = 2139                                                                   },
  { a = M.INTERRUPT,      c = L.ROGUE,                                                l = 14,                i = 1766                                                                   },
  { a = M.INTERRUPT,      c = L.SHAMAN,                                               l = 16,                i = 57994                                                                  },
  { a = M.INTERRUPT,      c = L.WARLOCK,                          p = P.FELHUNTER,                           i = 19647                                                                  },
  { a = M.INTERRUPT,      c = L.WARRIOR,                                              l = 38,                i = 6552                                                                   },
  -- Interrupt + silence
  { a = M.INTERRUPT,      c = L.HUNTER,                                                         tr=2,tc=1,   i = 34490                                                                  },
  { a = M.INTERRUPT,      c = L.PALADIN,                                              l = 54,                i = 96231                                                                  },
  { a = M.INTERRUPT,      c = L.PALADIN,      s = L.PROTPALADIN,                                             i = 31935                                                                  }, -- Specialization skill
  { a = M.INTERRUPT,      c = L.MAGE,                                                           tr=3,tc=3,   i = 102051                                                                 },
  --{ a = M.INTERRUPT,      c = L.ROGUE,                                                l = 40                                                                                            },
  --{ a = M.INTERRUPT,      c = L.WARRIOR,      s = L.PROTWARRIOR,                                  t = 12311                                                                             }, -- and 12958

  { a = M.INTERRUPT2,     c = L.DEATHKNIGHT,                                          l = 59,                i = 47476                                                                  },
  { a = M.INTERRUPT2,     c = L.DEATHKNIGHT,                                                    tr=3,tc=3,   i = 108194                                                                 },
  { a = M.INTERRUPT2,     c = L.DRUID,        s = L.BALANCE,                                      t = 78675                                                                             },
  { a = M.INTERRUPT2,     c = L.DRUID,        s = L.FERAL,                            l = 22,                i = 80964                                                                  },
  { a = M.INTERRUPT2,     c = L.DRUID,        s = L.FERAL,                            l = 22,                i = 80965                                                                  },
  { a = M.INTERRUPT2,     c = L.PRIEST,       s = L.SHADOW,                                       t = 15487                                                                             },
  { a = M.INTERRUPT2,     c = L.HUNTER,                           p = P.NETHERRAY,                           i = 50479                                                                  },
  { a = M.INTERRUPT2,     c = L.HUNTER,                           p = P.MOTH,                                i = 50318                                                                  },

  { a = M.INTERRUPT2,                                                                                        i = 25046,             display = L.BLOODELF_ROGUE,                         e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        i = 28730,             display = L.BLOODELF_CASTER,                        e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        i = 50613,             display = L.BLOODELF_DEATHKNIGHT,                   e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        i = 69179,             display = L.BLOODELF_WARRIOR,                       e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        i = 80483,             display = L.BLOODELF_HUNTER,                        e = R.BLOODELF },
  { a = M.INTERRUPT2,     c = L.WARRIOR,                                                        tr=3,tc=3,   i = 102060                                                                 },

  { a = C.CCBEASTS,       c = L.DRUID,        s = L.BALANCE,                          l = 48,                i = 2637                                                                   },
  { a = C.CCBEASTS,       c = L.DRUID,        s = L.RESTODRUID,                       l = 48,                i = 2637                                                                   },
  { a = C.CCDRAGONKIN,    c = L.DRUID,        s = L.BALANCE,                          l = 48,                i = 2637                                                                   },
  { a = C.CCDRAGONKIN,    c = L.DRUID,        s = L.RESTODRUID,                       l = 48,                i = 2637                                                                   },

  { a = C.CCBEASTS,       c = L.MAGE,                                                 l = 14,                i = 118                                                                    },
  { a = C.CCHUMANOIDS,    c = L.MAGE,                                                 l = 14,                i = 118                                                                    },

  { a = C.CCDEMONS,       c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },
  { a = C.CCDRAGONKIN,    c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },
  { a = C.CCGIANTS,       c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },
  { a = C.CCHUMANOIDS,    c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },
  { a = C.CCUNDEAD,       c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },

  { a = C.CCUNDEAD,       c = L.PRIEST,                                               l = 32,                i = 9484                                                                   },
  { a = C.CCHUMANOIDS,    c = L.PRIEST,                                               l = 38,                i = 605                                                                    },

  { a = C.CCBEASTS,       c = L.ROGUE,                                                l = 10,                i = 6770                                                                   },
  { a = C.CCDEMONS,       c = L.ROGUE,                                                l = 10,                i = 6770                                                                   },
  { a = C.CCDRAGONKIN,    c = L.ROGUE,                                                l = 10,                i = 6770                                                                   },
  { a = C.CCHUMANOIDS,    c = L.ROGUE,                                                l = 10,                i = 6770                                                                   },

  { a = C.CCELEMENTALS,   c = L.SHAMAN,                                               l = 64,                i = 76780                                                                  },

  { a = C.CCBEASTS,       c = L.SHAMAN,                                               l = 80,                i = 51514                                                                  },
  { a = C.CCHUMANOIDS,    c = L.SHAMAN,                                               l = 80,                i = 51514                                                                  },

  { a = C.CCDEMONS,       c = L.WARLOCK,                                              l = 32,                i = 710                                                                    },
  { a = C.CCDEMONS,       c = L.WARLOCK,                                              l = 30,                i = 1098                                                                   },
  { a = C.CCELEMENTALS,   c = L.WARLOCK,                                              l = 32,                i = 710                                                                    },

  { a = C.CCHUMANOIDS,    c = L.WARLOCK,                          p = P.SUCCUBUS,                            i = 6358                                                                   },

  { a = C.CCANY,          c = L.HUNTER,                                               l = 28,                i = 1499                                                                   },

  { a = M.ROOT,           c = L.DRUID,        s = L.BALANCE,                          l = 8,                 i = 339                                                                    },
  { a = M.ROOT,           c = L.DRUID,        s = L.RESTODRUID,                       l = 8,                 i = 339                                                                    },

  { a = M.SNARE,          c = L.DEATHKNIGHT,                                          l = 58,                i = 45524                                                                  },
--  { a = M.SNARE,          c = L.DRUID,        s = L.FERAL,                                        t = 48483                                                                             }, FIXME what was this?
  { a = M.SNARE,          c = L.DRUID,                                                          tr=3,tc=1,   i = 106707                                                                 },
  { a = M.SNARE,          c = L.HUNTER,                                               l = 8,                 i = 5116                                                                   },
  --{ a = M.SNARE,          c = L.MAGE,                                                 l = 56                                                                                            }, -- Frostfire Bolt
  { a = M.SNARE,          c = L.MAGE,                                                 l = 4,                 i = 116                                                                    }, -- Frost Bolt
  --{ a = M.SNARE,          c = L.MAGE,         s = L.ARCANE,                                       t = 31589                                                                             },
  { a = M.SNARE,          c = L.PALADIN,                                                         tr=2,tc=3,  i = 110301                                                                 },
  { a = M.SNARE,          c = L.PRIEST,       s = L.SHADOW,                                       t = 15407                                                                             },
  { a = M.SNARE,          c = L.ROGUE,                                                l = 20,                i = 3408                                                                   }, -- Crippling poison
  --{ a = M.SNARE,          c = L.ROGUE,                                                l = 62                                                                                            }, -- Deadly Throw
  --{ a = M.SNARE,          c = L.ROGUE,        s = L.ASSASINATION,                                 t = 51625                                                                             }, -- and 51626, Deadly Brew
  --{ a = M.SNARE,          c = L.ROGUE,        s = L.COMBAT,                                       t = 31124                                                                             }, -- and 31126, Blade Twisting
  --{ a = M.SNARE,          c = L.ROGUE,        s = L.SUBTLETY,                                     t = 51692                                                                             }, -- and 51686, Waylay
  { a = M.SNARE,          c = L.SHAMAN,                                               l = 22,                i = 8056                                                                   }, -- Frostshock!
  --{ a = M.SNARE,          c = L.SHAMAN,       s = L.ENHANCEMENT,                      l = 26                                                                                            }, -- Frostbrand Weapon
  { a = M.SNARE,          c = L.WARLOCK,      s = L.AFFLICTION,                                   t = 18223                                                                             },
  { a = M.SNARE,          c = L.WARRIOR,                                              l = 36,                i = 1715                                                                   },

  { a = M.AOESNARE,       c = L.DRUID,        s = L.BALANCE,                          l = 84,                i = 78777                                                                  },
  { a = M.AOESNARE,       c = L.HUNTER,                                               l = 46,                i = 13809                                                                  },
  { a = M.AOESNARE,       c = L.HUNTER,       s = L.MARKSMAN,                         l = 30,     t = 35102                                                                             }, -- And 35102
  { a = M.AOESNARE,       c = L.SHAMAN,                                               l = 18,                i = 2484                                                                   },
  { a = M.AOESNARE,       c = L.WARRIOR,                                                         tr=3,tc=2,  i = 12323                                                                  },

  -- checked from here
  { a = D.SUNDER,         c = L.DRUID,                                                l = 24,                i = 770                                                                    },
  { a = D.SUNDER,         c = L.HUNTER,                           p = P.TALLSTRIDER,  l = 20,                i = 50285                                                                  },
  { a = D.SUNDER,         c = L.HUNTER,                           p = P.RAPTOR,       l = 20,                i = 50498                                                                  },
  { a = D.SUNDER,         c = L.ROGUE,                                                l = 36,                i = 8647                                                                   },
  { a = D.SUNDER,         c = L.WARRIOR,      s = L.PROTWARRIOR,                      l = 18,                i = 7386                                                                   },

  { a = D.PHYSDMGTAKEN,   c = L.DEATHKNIGHT,  s = L.FROSTDK,                          l = 66,                i = 81328                                                                  },
  { a = D.PHYSDMGTAKEN,   c = L.DEATHKNIGHT,  s = L.UNHOLY,                           l = 68,                i = 51160                                                                  },
  { a = D.PHYSDMGTAKEN,   c = L.HUNTER,                           p = P.BOAR,         l = 20,                i = 35290                                                                  },
  { a = D.PHYSDMGTAKEN,   c = L.HUNTER,                           p = P.RAVAGER,      l = 20,                i = 50518                                                                  },
  { a = D.PHYSDMGTAKEN,   c = L.HUNTER,       s = L.BEASTMASTER,  p = P.RHINO,        l = 20,                i = 57386                                                                  },
  { a = D.PHYSDMGTAKEN,   c = L.HUNTER,       s = L.BEASTMASTER,  p = P.WORM,         l = 20,                i = 55749                                                                  },
  { a = D.PHYSDMGTAKEN,   c = L.PALADIN,      s = L.RETRIBUTION,                      l = 28,     t = 111529                                                                            },
  { a = D.PHYSDMGTAKEN,   c = L.WARRIOR,      s = L.ARMS,                             l = 81,     t = 86346                                                                             },
  { a = D.PHYSDMGTAKEN,   c = L.WARRIOR,      s = L.FURY,                             l = 81,     t = 86346                                                                             },

  { a = D.SPELLDMGTAKEN,  c = L.HUNTER,                           p = P.DRAGONHAWK,   l = 20,                i = 34889                                                                  },
  { a = D.SPELLDMGTAKEN,  c = L.HUNTER,                           p = P.WINDSERPENT,  l = 20,                i = 24844                                                                  },
  { a = D.SPELLDMGTAKEN,  c = L.ROGUE,                                                l = 64,     t = 58410                                                                             },
  { a = D.SPELLDMGTAKEN,  c = L.WARLOCK,                                              l = 52,                i = 1490                                                                   },

  { a = D.WEAKBLOWS,      c = L.DEATHKNIGHT,  s = L.BLOOD,                            l = 68,                i = 81132                                                                  },
  { a = D.WEAKBLOWS,      c = L.DRUID,        s = L.FERAL,                            l = 28,     t = 106830                                                                            },
  { a = D.WEAKBLOWS,      c = L.DRUID,        s = L.FERAL,                            l = 28,     t = 77758                                                                             },
  { a = D.WEAKBLOWS,      c = L.HUNTER,                           p = P.BEAR,         l = 20,                i = 50256                                                                  },
  { a = D.WEAKBLOWS,      c = L.HUNTER,                           p = P.CARRIONBIRD,  l = 20,                i = 24423                                                                  },
  { a = D.WEAKBLOWS,      c = L.MONK,         s = L.BREWMASTER,                       l = 11,     t = 121253                                                                            },
  { a = D.WEAKBLOWS,      c = L.PALADIN,      s = L.PROTPALADIN,                      l = 20,     t = 53595                                                                             },
  { a = D.WEAKBLOWS,      c = L.PALADIN,      s = L.RETRIBUTION,                      l = 20,     t = 53595                                                                             },
  { a = D.WEAKBLOWS,      c = L.SHAMAN,                                               l = 5,                 i = 8042                                                                   },
  { a = D.WEAKBLOWS,      c = L.WARRIOR,                                              l = 20,                i = 6343                                                                   },

  { a = D.CASTSPD,        c = L.DEATHKNIGHT,                                          l = 83,                i = 73975                                                                  },
  { a = D.CASTSPD,        c = L.HUNTER,                           p = P.SPOREBAT,     l = 20,                i = 50274                                                                  },
  { a = D.CASTSPD,        c = L.HUNTER,                           p = P.FOX,          l = 20,                i = 90314                                                                  },
  { a = D.CASTSPD,        c = L.HUNTER,                           p = P.GOAT,         l = 20,                i = 126402                                                                 },
  { a = D.CASTSPD,        c = L.HUNTER,       s = L.BEASTMASTER,  p = P.COREHOUND,    l = 20,                i = 58604                                                                  },
  { a = D.CASTSPD,        c = L.MAGE,         s = L.ARCANE,                           l = 36,     t = 31589                                                                             },
  { a = D.CASTSPD,        c = L.ROGUE,                                                l = 28,                i = 5761                                                                   },
  { a = D.CASTSPD,        c = L.WARLOCK,                                              l = 17,                i = 109466                                                                 },

  { a = D.MORTALSTRIKE,   c = L.HUNTER,                                               l = 40,                i = 82654                                                                  },
  { a = D.MORTALSTRIKE,   c = L.HUNTER,       s = L.BEASTMASTER,  p = P.DEVILSAUR,    l = 20,                i = 54680                                                                  },
  { a = D.MORTALSTRIKE,   c = L.MONK,         s = L.WINDWALKER,                       l = 56,     t = 107428                                                                            },
  { a = D.MORTALSTRIKE,   c = L.ROGUE,                                                l = 30,                i = 8679                                                                   },
  { a = D.MORTALSTRIKE,   c = L.WARRIOR,      s = L.ARMS,                             l = 10,     t = 12294                                                                             },
  { a = D.MORTALSTRIKE,   c = L.WARRIOR,      s = L.FURY,                             l = 18,     t = 100130                                                                            },
  -- checked to here

  { a = M.KNOCKBACK,      c = L.DRUID,        s = L.BALANCE,                                    tr=3,tc=3,   i = 132469                                                                 },
  { a = M.KNOCKBACK,      c = L.SHAMAN,       s = L.ELEMENTAL,                                               i = 51490                                                                  }, -- Specialization skill
  { a = M.KNOCKBACK,      c = L.WARLOCK,                          p = P.SUCCUBUS,                            i = 6360                                                                   },
}

for _, entry in ipairs (data.table) do
  -- Common display format unless overridden in the table above
  if not entry.display then 
    local disp = entry.c
    if entry.s then disp = disp..SPEC..entry.s end
    if entry.p then disp = disp..L.PET..entry.p end
    entry.display = disp
  end
end

