-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

local RC = RaidChecklist
local RCData = RaidChecklistData
local L = RaidChecklistStrings

-- Map from GUID to name
RC.provider_names = {}

-- Map from (de)buff key to list of GUIDs currently providing said de(buff)
RC.providers = RCData:create_provider_table ()

local function remove_provider (rguid)
  -- Iterate over all the (de)buffs
  for key,guidlist in pairs (RC.providers) do
    guidlist[rguid] = nil
  end
  if (RC.OnRefresh) then RC:OnRefresh () end
end

local function add_provider (buff, guid)
  RC.providers[buff][guid] = 3 -- "Provided"
  if (RC.OnRefresh) then RC:OnRefresh () end
end

local function update_missing ()
  local guids = RC.is:QueuedInspections ()
  local names
  for i,guid in ipairs (guids) do
    local unit = RC.is:GuidToUnit (guid)
    if unit then
      local name = UnitName (unit)
      names = names and names..", "..name or name
    end
  end
  return names
end


function RC:OnInspectUpdate (event, guid, unit, info)
  -- Update the talent status
  update_missing ()

  -- Clear any previous entries for this guid
  remove_provider (guid)

  -- Do matching
  local spec = RCSpecName (info.global_spec_id)
  local level = UnitLevel (unit)
  local class = L[info.class or UNKNOWN]
  local _, race = UnitRace (unit)
  for _, entry in ipairs (RCData.table) do
    if entry.c == class then
      local match = true
      if match and entry.p then match = false end -- Pets are handled elsewhere
      if match and entry.l and level < entry.l then match = false end
      if match and entry.s and spec ~= entry.s then match = false end
      if match and entry.tr and entry.tc then
        local row = info.talents[entry.tr]
        if row and row[entry.tc] and not row[entry.tc].selected then match = false end
      end
      if match then add_provider (entry.a, guid) end
    end
    if entry.e == race then add_provider (entry.a, guid) end
  end
  --print ("RC: updated talents for GUID "..(guid or "nil")..", unit "..unit)
end


function RC:OnInspectAdd (event, guid, unit, info)
  local fullname = (info.realm and info.name.."-"..info.realm) or info.name
if not info.class then print(fullname) end
  if not fullname then return end -- "shouldn't happen"

  RC.provider_names[guid] = ClassColorString (info.class or UNKNOWN)..fullname.."|r"
  -- Update based on class/level alone to start off with. Better than waiting until in inspect range!
  RC:OnInspectUpdate ("fake", guid, unit, info)

  -- We don't always seem to get a UNIT_PET event, so fake one to catch any initial pets
  RC.button:UNIT_PET(RC.is:GuidToUnit (guid))

  --print ("RC: discovered provider GUID "..(guid or "nil")..", named "..fullname)
end


function RC:OnInspectRemove (event, guid)
  -- Update the talent status
  update_missing ()

  remove_provider (guid)
  RC.provider_names[guid] = nil
  --print ("RC: removed provider GUID "..(guid or "nil"))
end


RC.is = LibStub ("LibGroupInSpecT-1.0")
RC.is.RegisterCallback (RC, "GroupInSpecT_Add", "OnInspectAdd")
RC.is.RegisterCallback (RC, "GroupInSpecT_Update", "OnInspectUpdate")
RC.is.RegisterCallback (RC, "GroupInSpecT_Remove", "OnInspectRemove")

-- Refresh on roster updates
local RCB = RC.button
function RCB:GROUP_ROSTER_UPDATE()
  RC:OnRefresh ()
end
RCB:RegisterEvent("GROUP_ROSTER_UPDATE")

